@@ -1,7 +1,7 @@
 package Dist::Zilla::Plugin::Rinci::AddPrereqs;
 
-our $DATE = '2018-06-11'; # DATE
-our $VERSION = '0.142'; # VERSION
+our $DATE = '2019-12-28'; # DATE
+our $VERSION = '0.143'; # VERSION
 
 use 5.010001;
 use strict;
@@ -82,14 +82,28 @@ sub _add_prereqs_from_func_meta {
             }
             $e = $arg_spec->{'x.completion'};
             if ($e && $cli_info) {
-                die "x.completion must be an array" unless ref($e) eq 'ARRAY';
-                my $pkg = "Perinci::Sub::XCompletion::$e->[0]";
+                my $xcomp_name;
+                if (ref $e eq 'CODE') {
+                    # can't do anything about it for now
+                } elsif (ref $e eq 'ARRAY') {
+                    $xcomp_name = $e->[0];
+                } else {
+                    $xcomp_name = $e;
+                }
+                my $pkg = "Perinci::Sub::XCompletion::$xcomp_name";
                 $self->_add_prereq($pkg => version_from_pmversions($pkg) // 0);
             }
             $e = $arg_spec->{'x.element_completion'};
             if ($e && $cli_info) {
-                die "x.element_completion must be an array" unless ref($e) eq 'ARRAY';
-                my $pkg = "Perinci::Sub::XCompletion::$e->[0]";
+                my $xcomp_name;
+                if (ref $e eq 'CODE') {
+                    # can't do anything about it for now
+                } elsif (ref $e eq 'ARRAY') {
+                    $xcomp_name = $e->[0];
+                } else {
+                    $xcomp_name = $e;
+                }
+                my $pkg = "Perinci::Sub::XCompletion::$xcomp_name";
                 $self->_add_prereq($pkg => version_from_pmversions($pkg) // 0);
             }
 
@@ -220,7 +234,7 @@ Dist::Zilla::Plugin::Rinci::AddPrereqs - Add prerequisites from Rinci metadata
 
 =head1 VERSION
 
-This document describes version 0.142 of Dist::Zilla::Plugin::Rinci::AddPrereqs (from Perl distribution Dist-Zilla-Plugin-Rinci-AddPrereqs), released on 2018-06-11.
+This document describes version 0.143 of Dist::Zilla::Plugin::Rinci::AddPrereqs (from Perl distribution Dist-Zilla-Plugin-Rinci-AddPrereqs), released on 2019-12-28.
 
 =head1 SYNOPSIS
 
@@ -295,7 +309,7 @@ perlancar <perlancar@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2018, 2016, 2015 by perlancar@cpan.org.
+This software is copyright (c) 2019, 2018, 2016, 2015 by perlancar@cpan.org.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
